import requests,json,re,csv,os,subprocess,urllib2,argparse,getpass
from pprint import pprint
from os.path import expanduser
from urllib2 import Request, urlopen
from clip_idlist import idlist
from clip_geojson import geojsonc
from cli_aoi2json import aoijson
from clip_json import jsonc
from clip_download import downloadclips
from cli_sorter import sort
os.chdir(os.path.dirname(os.path.realpath(__file__)))
def planet_key_entry():
    try:
        subprocess.call('planet init',shell=True)
    except Exception as e:
        print(e)
def planet_key_from_parser(args):
    planet_key_entry()
def planet_quota():
    try:
        subprocess.call('planet_quota.py',shell=True)
    except Exception as e:
        print(e)
def planet_quota_from_parser(args):
    planet_quota()
def aoijson_from_parser(args):
    aoijson(start=args.start,end=args.end,cloud=args.cloud,inputfile=args.inputfile,geo=args.geo,loc=args.loc)
def activate_from_parser(args):
    aoi_json=str(args.aoi)
    action_planet=str(args.action)
    asset_type=str(args.asset)
    subprocess.call("python download.py --query "+'"'+aoi_json+'" '+"--activate "+asset_type,shell=True)
##    try:
##        os.system("python download.py --query "+args.aoi+" --"+args.action+" "+asset_type)
##    except Exception:
##        print(' ')
def idlist_from_parser(args):
    idlist(aoi=args.aoi,item_asset=args.asset)
def geojsonc_from_parser(args):
    geojsonc(path=args.path,item=args.item,asset=args.asset)
def jsonc_from_parser(args):
    jsonc(path=args.path,item=args.item,asset=args.asset)
def downloadclips_from_parser(args):
    downloadclips(filepath=args.dir)
def sort_from_parser(args):
    sort(zipped=args.zipped,unzipped=args.unzipped)

spacing="                               "
def main(args=None):
    parser = argparse.ArgumentParser(description='Planet Clip Tools CLI')

    subparsers = parser.add_subparsers()

    parser_pp3 = subparsers.add_parser(' ', help='-------------------------------------------')
    parser_P2 = subparsers.add_parser(' ', help='-----Choose from Planet Clip Tools-----')
    parser_pp4 = subparsers.add_parser(' ', help='-------------------------------------------')

    parser_planet_key = subparsers.add_parser('apikey', help='Enter your planet API Key')
    parser_planet_key.set_defaults(func=planet_key_from_parser)

    parser_planet_quota = subparsers.add_parser('quota', help='Prints your Planet Quota Details')
    parser_planet_quota.set_defaults(func=planet_quota_from_parser)

    parser_aoijson=subparsers.add_parser('aoijson',help='Tool to convert KML, Shapefile,WKT,GeoJSON or Landsat WRS PathRow file to AreaOfInterest.JSON file with structured query for use with Planet API 1.0')
    parser_aoijson.add_argument('--start', help='Start date in YYYY-MM-DD?')
    parser_aoijson.add_argument('--end', help='End date in YYYY-MM-DD?')
    parser_aoijson.add_argument('--cloud', help='Maximum Cloud Cover(0-1) representing 0-100')
    parser_aoijson.add_argument('--inputfile',help='Choose a kml/shapefile/geojson or WKT file for AOI(KML/SHP/GJSON/WKT) or WRS (6 digit RowPath Example: 023042)')
    parser_aoijson.add_argument('--geo', default='./map.geojson',help='map.geojson/aoi.kml/aoi.shp/aoi.wkt file')
    parser_aoijson.add_argument('--loc', help='Location where aoi.json file is to be stored')
    parser_aoijson.set_defaults(func=aoijson_from_parser)
    
    parser_activate=subparsers.add_parser('activate',help='Tool to query and/or activate Planet Assets')
    parser_activate.add_argument('--aoi', help='Choose aoi.json file created earlier')
    parser_activate.add_argument('--action', help='choose between check/activate')
    parser_activate.add_argument('--asset',help='Choose between planet asset types (PSOrthoTile analytic/PSOrthoTile analytic_dn/PSOrthoTile visual/PSScene4Band analytic/PSScene4Band analytic_dn/PSScene3Band analytic/PSScene3Band analytic_dn/PSScene3Band visual/REOrthoTile analytic/REOrthoTile visual')
    parser_activate.set_defaults(func=activate_from_parser)

    parser_idlist = subparsers.add_parser('idlist', help='Allows users to generate an id list for the selected item and asset type for example item_asset= PSOrthoTile analytic/PSScene3Band visual. This is used with the clip tool')
    parser_idlist.add_argument('--aoi', help='Input path to the structured json file from which we will generate the clips',default=None)
    parser_idlist.add_argument('--asset', help='Choose from asset type for example:"PSOrthoTile analytic"|"REOrthoTile analytic"',default=None)
    parser_idlist.set_defaults(func=idlist_from_parser)

    parser_geojsonc = subparsers.add_parser('geojsonc', help='Allows users to batch submit clipping request to the Planet Clip API using geometry in geojson file')
    parser_geojsonc.add_argument('--path', help='Path to the geojson file including filename',default=None)
    parser_geojsonc.add_argument('--item', help='Choose from item type for example:"PSOrthoTile","REOrthoTile"',default=None)
    parser_geojsonc.add_argument('--asset', help='Choose from asset type for example: "visual","analytic"',default=None)
    parser_geojsonc.set_defaults(func=geojsonc_from_parser)

    parser_jsonc = subparsers.add_parser('jsonc', help='Allows users to batch submit clipping request to the Planet Clip API using geometry in structured json file. This is preferred because the structured JSON allows the activate tool to stream line asset ids being requested and to extract geometry from the same file')
    parser_jsonc.add_argument('--path', help='Path to the json file including filename',default=None)
    parser_jsonc.add_argument('--item', help='Choose from item type for example:"PSOrthoTile","REOrthoTile"',default=None)
    parser_jsonc.add_argument('--asset', help='Choose from asset type for example: "visual","analytic"',default=None)
    parser_jsonc.set_defaults(func=jsonc_from_parser)

    parser_downloadclips = subparsers.add_parser('downloadclips', help='Allows users to batch download clipped assets post computation using a directory path(Requires you to first activate and run geojson or json tool)')
    parser_downloadclips.add_argument('--dir', help='Output directory to save the assets. All files are zipped and include metadata',default=None)
    parser_downloadclips.set_defaults(func=downloadclips_from_parser)

    parser_sort = subparsers.add_parser('sort', help='Allows users to unzip downloaded files to new folder and sorts into images and metadata')
    parser_sort.add_argument('--zipped', help='Folder containing downloaded clipped files which are zipped',default=None)
    parser_sort.add_argument('--unzipped', help='Folder where you want your files to be unzipped and sorted',default=None)
    parser_sort.set_defaults(func=sort_from_parser)

    args = parser.parse_args()

    args.func(args)

if __name__ == '__main__':
    main()



import subprocess
import os
import json
from pprint import pprint
import argparse
import sys
import time
import os
import csv
import shapefile
import string
p1='{"config": [{"field_name": "geometry", "config": {"type": "Polygon","coordinates":'
p2='}, "type": "GeometryFilter"}, {"field_name": "gsd", "config": {"gte":1,"lte":9.99}, "type": "RangeFilter"}, {"field_name": "acquired", "config": {"gte":"'
p3='T04:00:00.000Z","lte":"'
p4='T03:59:59.999Z"}, "type": "DateRangeFilter"}, {"field_name": "cloud_cover", "config": {"gte":0'
p5=',"lte":'
p6='}, "type": "RangeFilter"}], "type": "AndFilter"}'

dir_path = os.path.dirname(os.path.realpath(__file__))

def aoijson(start,end,cloud,inputfile,geo,loc):
    if inputfile == 'KML':
        os.system("python kml_aoi.py --start "+start+" --end "+end+" --cloud "+cloud+" --geo "+geo+" --loc "+loc)
    elif inputfile=='WRS':
         with open(dir_path+'./wrs_grid.csv', 'rb') as f:
            reader = csv.reader(f)
            for row in reader:
                if row[13]== geo:
                    a=str(row[14])
                    strpd=a.split(':')[3].strip('}')
                    filenames = p1+strpd+p2+str(start)+p3+str(end)+p4+p5+str(cloud)+p6
                    with open(loc+'./aoi.json', 'w') as outfile:
                        outfile.write(filenames)
                        outfile.close()
    elif inputfile == 'GJSON':
        with open(geo) as insert,open(os.path.join(dir_path,"aoi.json")) as jbase:
            geombase=json.load(jbase)
            geomloader = json.load(insert)
            cinsert= geomloader['features'][0]['geometry']['coordinates']
            cgeom=(geombase['config'][0]['config']['coordinates'])
            geombase['config'][0]['config']['coordinates']=cinsert #coordinate insert
            geombase['config'][2]['config']['gte']=str(start)+"T04:00:00.000Z" #change start date
            geombase['config'][2]['config']['lte']=str(end)+"T03:59:59.999Z" #change end date
            geombase['config'][3]['config']['gte']=0#change cloud minima default=0
            geombase['config'][3]['config']['lte']=float(cloud)# change cloud maxima
            with open(loc+"./aoi.json", 'w') as f:
                f.write(json.dumps(geombase))
            print("New structured JSON has been created at "+str(os.path.join(loc,"aoi.json")))      
    elif inputfile == 'SHP':
        reader = shapefile.Reader(geo)
        fields = reader.fields[1:]
        field_names = [field[0] for field in fields]
        buffer = []
        for sr in reader.shapeRecords():
            atr = dict(zip(field_names, sr.record))
            geom = sr.shape.__geo_interface__
            buffer.append(dict(type="Feature", \
            geometry=geom, properties=atr))
            geom2=str(geom).replace("(",'[')
            geom3=str(geom2).replace(")",']')
            geom4=str(geom3).replace(",]",']')
            #print(geom5)
        # write the GeoJSON file
        with open(loc+'./int.geojson','w') as csvfile:
            writer=csv.writer(csvfile)
            writer.writerow([str(geom4)])
        raw= open(loc+'./int.geojson')
        for line in raw:
            fields=line.strip().split(":")[2]
            f2=fields.strip().split("}")[0]
            filenames = p1+f2+p2+str(start)+p3+str(end)+p4+p5+str(cloud)+p6
        with open(loc+'./aoi.json', 'w') as outfile:
            outfile.write(filenames)
            outfile.close()
    elif inputfile == 'WKT':
        raw= open(geo)
        for line in raw:
            l1=str(line).replace("POLYGON ((","[[[")
            l2=str(l1).replace("))","]]]")
            l3=str(l2).replace(", ","],[")
            l4=str(l3).replace(" ",",")
            filenames = p1+l4+p2+str(start)+p3+str(end)+p4+p5+str(cloud)+p6
        with open(loc+'./aoi.json', 'w') as outfile:
            outfile.write(filenames)
            outfile.close()



import os,shutil,zipfile,subprocess
os.chdir(os.path.dirname(os.path.realpath(__file__)))
def sort(zipped,unzipped):
    if not os.path.exists(os.path.join(unzipped,"images")):
        os.mkdir(os.path.join(unzipped,"images"))
    if not os.path.exists(os.path.join(unzipped,"metadata")):
        os.mkdir(os.path.join(unzipped,"metadata"))
    ziplist=os.listdir(zipped)
    for zipfiles in ziplist:
        file_name = os.path.join(zipped,zipfiles) # get full path of files
        zip_ref = zipfile.ZipFile(file_name) # create zipfile object
        zip_ref.extractall(unzipped) # extract file to dir
        zip_ref.close() # close file
        print("Unzipping "+file_name)
    filelist=os.listdir(unzipped)
    for files in filelist:
        if files.endswith(".tif"):
            baseimage=os.path.basename(files)
            shutil.move(os.path.join(unzipped,files), os.path.join(unzipped,"images"))
            print("Moving "+baseimage)
        if files.endswith(".xml"):
            basemeta=os.path.basename(files)
            shutil.move(os.path.join(unzipped,files), os.path.join(unzipped,"metadata"))
            print("Moving "+basemeta)
    for zipfiles in ziplist:
        if zipfiles.endswith(".zip"):
            print("Deleting "+str(zipfiles))
            os.unlink(os.path.join(zipped,zipfiles))



import requests,json,re,csv,os,subprocess,urllib2,getpass,time,progressbar
from pprint import pprint
from os.path import expanduser
from urllib2 import Request, urlopen
from planet.api.utils import read_planet_json
os.chdir(os.path.dirname(os.path.realpath(__file__)))
planethome=os.path.dirname(os.path.realpath(__file__))
try:
        PL_API_KEY = read_planet_json()['key']
except:
        subprocess.call('planet init',shell=True)
CAS_URL='https://api.planet.com/compute/ops/clips/v1/'
headers = {'Content-Type': 'application/json',}
if not os.path.exists(os.path.join(planethome,'urllist.csv')):
        with open(os.path.join(planethome,'urllist.csv'),'w') as completed:
            writer=csv.writer(completed,delimiter=',',lineterminator='\n')
with open(os.path.join(planethome,"urllist.csv")) as f:
    csv_f = csv.reader(f)
    value = len(list(csv_f))
def downloadclips(filepath=None):
    with open(os.path.join(planethome,"urllist.csv")) as f:
        csv_f = csv.reader(f)
        for i,row in enumerate(csv_f):
            URL=str(row).replace("'","").replace("[","").replace("]","")
            downlink = requests.get(url=URL, auth=(PL_API_KEY, ''))
            if downlink.status_code==200:
                content=downlink.json()
                item_id=(content['targets'][0]['item_id'])
                item_typ=(content['targets'][0]['item_type'])
                asset_typ=(content['targets'][0]['asset_type'])
                try:
                    if content['state']=='running':
                        while content['state']=='running':
                            bar = progressbar.ProgressBar()
                            for i in bar(range(60)):
                                time.sleep(1)
                            downlink = requests.get(url=URL, auth=(PL_API_KEY, ''))
                            if downlink.status_code==200:
                                content=downlink.json()
                    elif content['state']=='succeeded':
                        filelink = urllib2.urlopen(str(content['_links']['results'][0]))
                        filename=item_id+"_"+item_typ+"_"+asset_typ+".zip"
                        ov=os.path.join(filepath,filename)
                        if not os.path.exists(ov):
                            with open(ov, "wb") as code:
                                code.write(filelink.read())
                            print("Downloading: "+str(i+1)+" of "+str(value)+" "+(item_id+"_"+item_typ+"_"+asset_typ))
                        else:
                            print("asset exists..Skipping "+str(filename))

                    else:
                        print("Clip Function still " +content['state']+" "+str(filename))
                except Exception as e:
                    print e
            else:
                print("Got Download Return Error Code: "+str(downlink.status_code))



import requests,json,re,csv,os,subprocess,urllib2,getpass
from pprint import pprint
from os.path import expanduser
from urllib2 import Request, urlopen
from os.path import expanduser
from planet.api.utils import read_planet_json
os.chdir(os.path.dirname(os.path.realpath(__file__)))
planethome=os.path.dirname(os.path.realpath(__file__))
try:
        PL_API_KEY = read_planet_json()['key']
except:
        subprocess.call('planet init',shell=True)

CAS_URL='https://api.planet.com/compute/ops/clips/v1/'
headers = {'Content-Type': 'application/json',}
def geojsonc(path=None,item=None,asset=None):
    with open(path) as f, open(os.path.join(planethome,"idl.csv"),'r') as f2,open(os.path.join(planethome,"urllist.csv"),'wb') as csvfile:
        geomloader = json.load(f)
        geom=geomloader['features'][0]['geometry']
        reader = csv.DictReader(f2)
        writer=csv.DictWriter(csvfile,fieldnames=["url"], delimiter=',')
        writer.writeheader()
        for i, line in enumerate(reader):
            item_id = line['id_no']
            data = r'{"aoi": '+str(geom)+r',"targets": [{"item_id": '+'"'+item_id+'"'+r',"item_type": '+'"'+item+'"'+r',"asset_type": '+'"'+asset+'"'+r'}]}'
            data2=str(data).replace("'",'"').replace('u"','"')
            main=requests.post('https://api.planet.com/compute/ops/clips/v1/', headers=headers, data=data2, auth=(PL_API_KEY, ''))
            if main.status_code==202:
                URL=str(main.json()).split('_self')[1].split(',')[0].replace("': u","").replace(" ","").replace("'","")
                content=main.json()
                item_id=(content['targets'][0]['item_id'])
                item_typ=(content['targets'][0]['item_type'])
                asset_typ=(content['targets'][0]['asset_type'])
                print("Clipping: "+(item_id+"_"+item_typ+"_"+asset_typ))
                with open(os.path.join(planethome,"urllist.csv"),'a') as csvfile:
                    writer=csv.writer(csvfile,delimiter=',',lineterminator='\n')
                    writer.writerow([URL])
            elif main.status_code==429:
                time.sleep(2)
                URL=str(main.json()).split('_self')[1].split(',')[0].replace("': u","").replace(" ","").replace("'","")
                content=main.json()
                item_id=(content['targets'][0]['item_id'])
                item_typ=(content['targets'][0]['item_type'])
                asset_typ=(content['targets'][0]['asset_type'])
                print("Clipping: "+(item_id+"_"+item_typ+"_"+asset_typ))
                with open(os.path.join(planethome,"urllist.csv"),'a') as csvfile:
                    writer=csv.writer(csvfile,delimiter=',',lineterminator='\n')
                    writer.writerow([URL])
            else:
                print("Issues with: "+(item_id+"_"+item+"_"+asset)+" has error code "+str(main.status_code))



import sys
import os
import sys
import setuptools
from setuptools import find_packages
from setuptools.command.test import test as TestCommand
from distutils.version import StrictVersion
from setuptools import __version__ as setuptools_version

if StrictVersion(setuptools_version) < StrictVersion('38.3.0'):
    raise SystemExit(
        'Your `setuptools` version is old. '
        'Please upgrade setuptools by running `pip install -U setuptools` '
        'and try again.'
    )
def readme():
    with open('README.md') as f:
        return f.read()
setuptools.setup(
    name='pclip',
    version='0.2.2',
    packages={'pclip': ['aoi.json']},
    package_data={'pclip': ['aoi.json']},
    url='https://github.com/samapriya/Clip-Ship-Planet-CLI',
    install_requires=['planet>=1.1.0','psutil>=5.2.2','urllib3>=1.22','requests>=2.18.4','retrying>=1.3.3','pyshp>=1.2.12','progressbar2>=3.34.3',          'pypiwin32; platform_system == "Windows"','pywin32; platform_system == "Windows"'],
    license='Apache 2.0',
    long_description=open('README.rst').read(),
    classifiers=(
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Intended Audience :: Science/Research',
        'Natural Language :: English',
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.6',
        'Operating System :: OS Independent',
        'Topic :: Scientific/Engineering :: GIS',
    ),
    author='Samapriya Roy',
    author_email='samapriya.roy@gmail.com',
    description='Planet API Pipeline & Google Earth Engine Batch Assets Manager with Addons',
    entry_points={
        'console_scripts': [
            'pclip=pclip.pclip:main',
        ],
    },
)
